/********************************************************************************
 *               Conficker File & Registry Checker
 *
 *
 *
 * Copyright (C) 2009  Felix Leder & Tillmann Werner
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * 
 *             contact {leder,werner}@cs.uni-bonn.de
 *
 *******************************************************************************/


#include <windows.h>
#include <stdio.h>
#include "namegen.h"

#define AUTORUNPATH "Software\\Microsoft\\Windows\\CurrentVersion\\Run"

void PrintErrorMessage(char* prefix);

void print_creds() {
     printf("\n----------------------------------\n"	\
	    "Conficker File and Registry Checker\n"	\
	    "----------------------------------\n\n"	\
	    "Felix Leder, Tillmann Werner 2009\n"	\
            "{leder, werner}@cs.uni-bonn.de\n\n"	\
            "----------------------------------\n");
}


int try_deletion(char* filename){
  
  HANDLE file;

  file = CreateFile(filename,
		    GENERIC_READ,		    
		    FILE_SHARE_READ,		    
		    NULL,
		    OPEN_EXISTING,
		    FILE_ATTRIBUTE_NORMAL, 
		    NULL);
  if ((file == INVALID_HANDLE_VALUE) && (GetLastError()== ERROR_FILE_NOT_FOUND))
    return 0;

  if (file != INVALID_HANDLE_VALUE) CloseHandle(file);
  printf("[WARNING]: File \"%s\" found - possibly infected\n", filename);
    
  
  if (!DeleteFile(filename))
    PrintErrorMessage("Deletion failed");

  if (MoveFileEx(filename, NULL, MOVEFILE_DELAY_UNTIL_REBOOT)){
    printf("Trying deletion after reboot\n\n");
    return 1;
  }
  else
    PrintErrorMessage("Removal attempt failed");
  
  return 2;

}


int main(int argc, char* argv[]) {
  TCHAR sysdir[256];
  TCHAR namebuf[256];
  TCHAR fullname[512];
  
  if (!GetSystemDirectory(sysdir, sizeof(sysdir)-1)) {
    PrintErrorMessage("Cannot get system directory");
    return 1;
  }
  
  print_creds();
  
  printf("On this computer Conficker will be installed in...\n");
  
  //Conficker.A
  printf("\nConficker.A...\n");
  printf("DLL: unknown (random)\n");


  //Conficker.B
  printf("\nConficker.B...\n");

  if (getDllName(namebuf, sizeof(namebuf), CONFICKER_VARIANT_B)<0)
    fprintf(stderr, "Error: cannot get DLL name\n"); 
  else {

    snprintf(fullname, sizeof(fullname)-1, "%s\\%s.dll", sysdir, namebuf);
    printf("DLL: %s...", fullname);

    if (!try_deletion(fullname)) 
      printf("clean (non existent)\n");
  }

  if (getRegRunKey(namebuf, sizeof(namebuf), CONFICKER_VARIANT_B)<0)
    fprintf(stderr, "Error: cannot get DLL name\n"); 
  else {
    printf("Registry key: HKEY_LOCAL_MACHINE\\%s\\%s\n", AUTORUNPATH, namebuf);
    printf("Registry key: HKEY_CURRENT_USER\\%s\\%s\n", AUTORUNPATH, namebuf);
  }


  //Conficker.C
  printf("\nConficker.C...\n");
  if (getDllName(namebuf, sizeof(namebuf), CONFICKER_VARIANT_C)<0){
    fprintf(stderr, "Error: cannot get DLL name\n");
    
  }
  else {    
    snprintf(fullname, sizeof(fullname)-1, "%s\\%s.dll", sysdir, namebuf);
    printf("DLL: %s...", fullname);
    
    if (!try_deletion(fullname)) 
      printf("clean (non existent)\n");
  }

  if (getRegRunKey(namebuf, sizeof(namebuf), CONFICKER_VARIANT_C)<0)
    fprintf(stderr, "Error: cannot get DLL name\n"); 
  else {
    printf("Registry key: HKEY_LOCAL_MACHINE\\%s\\%s\n", AUTORUNPATH, namebuf);
    printf("Registry key: HKEY_CURRENT_USER\\%s\\%s\n", AUTORUNPATH, namebuf);
  }

  return 0;
}


void PrintErrorMessage(char* prefix) {
  LPVOID lpMsgBuf;

  FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
		 );

  printf("Error [%i] %s: %s\n", (int)GetLastError(), prefix, (char*) lpMsgBuf);

  // Free the buffer.
  LocalFree( lpMsgBuf );
  
}
